<?php
//
//  Copyright (c) 2022 NetApp, Inc.
//  All rights reserved.
//

// VERSION="1.0"

// burt1485006

// We accidentally allowed non-normalized initiator names (with upper case
// characters) into RDB. If an uppercase initiator and the same initiator with
// lowercase characters are mapped to the same LUN, scsit will panic if they
// are removed. The upgrade task that automatically fixes this problem will
// fail. Prior to upgrade, the customer needs to contact support to fix any
// duplicates that would create a conflict in RDB after normalization.
// Hopefully with this pre-upgrade error, customers will be able to plan the
// potential outage for this along with their upgrade for minimal disruption.

// We use similar steps to the plan we created for customer support to fix the
// problem. We skip the remediation steps and only detect the problem with this
// check.
//
// This is the full plan developed for the original burt customer. Steps 2 and
// 3 are intended to make it impossible to miss anything since that customer
// did not have this upgrade check available. If this upgrade task returns an
// error, we know steps 2 and 3 will fail and require the sblade fence at some
// point, but possibly not for every initiator.
//
// 1. Query for upper case initiators: igroup show -initiator *A*|*B*|*C*|*D*|*E*|*F*|*G*|*H*|*I*|*J*|*K*|*L*|*M*|*N*|*O*|*P*|*Q*|*R*|*S*|*T*|*U*|*V*|*W*|*X*|*Y*|*Z*
// 2. Use 'igroup remove' *without* -force to remove uppercase initiators and
//    re-add them in lowercase.
// 3. If you hit an error with the 'igroup remove', there is a map associated
//    with the initiator.
// 4. Run 'lun mapping show -fields initiator -initiator <upper case initiator
//    that couldn't be removed|same initiator, but tolower'ed>'. Note that this
//    query is not perfect. We really need a query that replaces all letters
//    with <lower|upper> on an individual letter basis, but SMF doesn't support
//    a query like that. If a maniacal config slips through this check, the
//    upgrade task will catch it and customer support will need to tolower the
//    unfiltered lun mapping list to find the duplicates. We could do that
//    here, but it would require processing the query in PHP after returning
//    all LUN maps, and that doesn't seem worth the performance hit on
//    potentially huge customer configs for a very unlikely case.
//   a. If this command does not show the same LUN more than once, the
//      initiator can be removed with -force and re-added. Note that the table
//      is in LUN order, so any duplicates will be adjacent.
//   b. If this command does show any duplicates, we'll need to raise the MCC
//      fence so updates do not reach scsit, and then we'll need to reboot to
//      reload scsit after correcting the duplicate.
//     i.   Create an sblade fence via debug smdb table. Note that on MCC
//          systems, the entry might already exist. If so, modify it to true and
//          then modify it back to false instead of create/delete.
//          'debug smdb table sblade_fence_rdb create -vsUuid <vsuuid> -is-fenced true'
//            OR
//          'debug smdb table sblade_fence_rdb modify -vsUuid <vsuuid> -is-fenced true'
//     ii.  Remove the duplicate with force.
//     iii. Delete the sblade fence entry. Or modify it back to false if it
//          existed prior. Note that you can leave the fence raised until
//          fixing all duplicate initiators.
//          'debug smdb table sblade_fence_rdb delete -vsUuid <vsuuid> -is-fenced true'
//            OR
//          'debug smdb table sblade_fence_rdb modify -vsUuid <vsuuid> -is-fenced false'
//     iv.  Reboot. Note that you can delay the reboot until fixing all
//          duplicate initiators. All nodes with a SAN LIF should be rebooted.

// First, find any initiators that have uppercase characters. The only way we
// can get a duplicate is if we have uppercase initiators.
$query = "SELECT initiator FROM igroup WHERE";
$query .= " initiator = *A*|*B*|*C*|*D*|*E*|*F*|*G*|*H*|*I*|*J*|*K*|*L*|*M*|*N*|*O*|*P*|*Q*|*R*|*S*|*T*|*U*|*V*|*W*|*X*|*Y*|*Z*";
$result = dotsql_query($query);
while ($row = dotsql_fetch_object($result)) {

    // Lists are returned comma delimited, so iterate each initiator one by one
    // to make duplicates easier to identify.
    $initiators = explode(",", $row->initiator);
    foreach ($initiators as $initiator) {
        if (!preg_match('/[A-Z]/', $initiator)) {
            // Ignore lowercase initiators if the igroup happens to have them.
            continue;
        }

        // For each uppercase initiator, see if we have a duplicate. The map
        // table is ordered (vserver, path), so duplicates will always be
        // adjacent.
        $map_query = "SELECT vserver, path FROM map WHERE initiators = $initiator|";

        // We found upper. Also query for the 'duplicate' lower version.
        $map_query .= strtolower($initiator);

        $map_result = dotsql_query($map_query);

        $last = NULL;
        while ($map_row = dotsql_fetch_object($map_result)) {
            if (!is_null($last) &&
                $last->vserver === $map_row->vserver &&
                $last->path === $map_row->path) {
                echo <<<MESSAGE
    <Alert>
        Duplicate initiators are mapped to the same LUN. Do not try to fix this problem. Using either the 'lun mapping delete' command or the 'lun igroup remove' command could lead to an outage. Contact technical support for assistance.
    </Alert>
    <Action>
        Contact technical support for assistance.
    </Action>
    <RC>error</RC>
MESSAGE;
                exit;
            }
            $last = $map_row;
        }
    }
}

exit;
?>
