<?php
//  Copyright (c) 2023 NetApp, Inc.
//  All rights reserved.

// NDU Compatibility for OpenSSH Key Exchange Algorithms.
// VERSION="1.0"
// This script checks whether SHA-1 Key Exchange Algorithms are enabled
// and issues warning if they are. NDU will not be allowed if no secure
// Key Exchange Algorithms are configured as unsecure Key Exchange
// Algorithms will be removed from SSH configuration during upgrade.
//
include 'mcc_util.php';

// All the validations only apply for major upgrades.
// Exit the validation in minor upgrades.
$options = getoptreq('', array("major:", "downgrade::"));
$majorupgrade = $options["major"];
if($majorupgrade == "false") {
    exit;
}

$ecvQuery = "SELECT generation,major FROM effective_cluster_version";
$result = dotsql_query($ecvQuery);
$row = dotsql_fetch_object($result);
if( $row ){
    $gen = $row->{'generation'};
    $major = $row->{'major'};
    if( !( $gen > 9 || ( $gen >= 9 && $major >= 11 ) ) ){
	exit;
    }
}

$vserver_name = " ";
$unsecure_kex_only = false;
$unsecure_kex_present = false;
$vserver_query = "SELECT id,vserver FROM vserver_by_name_data_admin";
$vserver_result = dotsql_query($vserver_query);
while($vserver_row = dotsql_fetch_object($vserver_result)){
    $vserver = $vserver_row->{'id'};
    $ssh_query = "SELECT key-exchange-algorithms FROM ssh WHERE vserver=$vserver";
    $ssh_result = dotsql_query($ssh_query);
    $ssh_row = dotsql_fetch_object($ssh_result);
    $kex_algorithms = $ssh_row->{'key-exchange-algorithms'};
    $kex_count = substr_count($kex_algorithms, ',') + 1;
    $weak_kex_count = substr_count($kex_algorithms, 'sha1');
    if($weak_kex_count > 0) {
	$unsecure_kex_present = true;
	if($weak_kex_count == $kex_count){
	    $unsecure_kex_only = true;
	    $vserver_name = $vserver_row->{'vserver'};
	    break;
	}
    }
}

$fips_enabled = false;
$fips_query = "SELECT is-fips-enabled FROM security_config_ui";
$fips_result = dotsql_query($fips_query);
$fips_row = dotsql_fetch_object($fips_result);
if($fips_row->{'is-fips-enabled'} == "true") {
    $fips_enabled = true;
}

if($unsecure_kex_only || $unsecure_kex_present) {
    echo "<Alert>";
    echo "Vserver found with SHA-1 Key Exchange Algorithm. ";
    echo "SHA-1 Key Exhange Algorithms are considered weak, ";
    echo "and will be removed upon upgrade.";
    echo "</Alert>";
    echo "<Action>";
    if($unsecure_kex_only) {
    echo "Add atleast one of the below secure Key Exchange ";
    echo "Algorithms to Vserver \"$vserver_name\" before retrying the upgrade: ";
    echo "diffie-hellman-group-exchange-sha256, ecdh-sha2-nistp256, ";
    echo "ecdh-sha2-nistp384, ecdh-sha2-nistp521";
    if(!($fips_enabled)) {
	echo ", curve25519-sha256";
    }
    }
    echo "</Action>";
    echo "<Advice>";
    echo "Ensure that all the Vservers are configured with secure Key ";
    echo "Exchange Algorithms.";
    echo "</Advice>";
    echo "<RC>";
    if($unsecure_kex_only) {
	echo "error";
    } else {
	echo "warning";
    }
    echo "</RC>";
}

exit;
?>
