<?php
//
// Copyright (c) 2014-2015 NetApp, Inc.
// All rights reserved.
//
// VERSION="1.0"
//
// This validation script checks for 'supported platform' in a software release.
// When a new Data ONTAP software is released, some board families or system
// models may be removed from the supported platform list.  When this happens,
// the 'supported_systems' list below needs to be updated.
//
// IMPORTANT: Please ensure the 'supported_systems' array is updated for
//            EVERY SOFTWARE RELEASE.
//
// Make sure there is corresponding changes in xxx_ident.c when removing the support for
// a platform

$supported_systems = array (
    // LML family (SB_XXI) supported model
    'XXI' =>   'MPOC, LML, FASLODI, FASMDRA, FASLVMR,'
             . 'FAS9040, FAS9060, FAS9080, FAS9000,'
             . 'AFF9040, AFF9060, AFF9080, AFF-A700',
    // Razor family (SB_XXII) supported model
    'XXII' =>  'Razor, FASRAZL, FASRAZXL,'
             . 'FAS2980, FAS2600, FAS8200,'
             . 'FAS2720, FAS2750,'
             . 'AFF-A300, AFF-A220, AFF-C190, AFF-A150, ASA-A150,'
             . 'DM3000H, DM5000H, DM7000H,'
             . 'DM5000F, DM7000F,'
             . 'ETERNUS HX2100, ETERNUS HX2200,'
             . 'ETERNUS AX2100, ETERNUS AX1100, ETERNUS AX1200, ETERNUS AX1200S',
    // Aurora family (SB_XXIII) supported model
    'XXIII' => 'AFF-A700s',
    // Apollo family (SB_XXIV) supported model
    'XXIV' =>  'Apollo, AFF-A800, AFF-C800, ASA-A800, ASA-C800',
    // Saber family (SB_XXV) supported model
    'XXV'   => 'AFF-A320, DM9000N, ETERNUS AX3100',
    // Katana family (SB_XXVII) supported model
    'XXVII' => 'FAS8300, FAS8700,'
            .  'AFF-A400, AFF-A600, AFF-C400, ASA-A400, ASA-C400,'
            .  'DM7100H, DM7100F, DG7000,'
            .  'ETERNUS HX6100, ETERNUS AX4100, ETERNUS AX4100S, ETERNUS AC4100, ETERNUS AC4100S',
    // Bandit family (SB_XXVIII) supported model
    'XXVIII' => 'AFF-A250, BANDIT_L, ASA-A250, ASA-C250,'
            .  'DM5100F, DG5000, ETERNUS AX2200, ETERNUS AX2200S, ETERNUS AC2100, ETERNUS AC2100S,'
            .  'FAS500f, AFF-C250',
    // Mendocino family (SB_XXIX) supported model
    'XXIX' => 'AFF-A900, FAS9500, ASA-A900',
    // Dagger2 family (SB_XXX) supported model
    'XXX'  => 'FAS2820, DM3010H, ETERNUS HX2300',
);


//
// Messages issued to user/customer.
//
$action_msg = "Use the \"cluster unjoin\" command to remove the unsupported system "
            . "from the cluster or upgrade the controller hardware before "
            . "installing the new software release.";
$advice_msg = "Remove the unsupported system from the cluster or upgrade the "
            . "controller hardware before installing the new software release.";
$error_msg = "The following nodes are not supported: ";


$vsim_family = 'NetApp VSim';
$vsa_family = 'Virtual Machine';
$error_detected = false;
$token_seperator = ",;\n\t";
$node_list = "";


function logDebugMessage($message, $log_error) {
    $dbgquery = "CALL add_scripting_log_message('validation_rules_platform_check: $message',
                 $log_error)";
    $result = dotsql_query($dbgquery);
}


/* Get board family and system model from SMF table 'system_node_info' */
$query = "SELECT system-name, system-machine-type, board-type, system-model FROM system_node_info";
$result = dotsql_query($query);

/* Perform platform support validation for all nodes found in a cluster */
while ($row = dotsql_fetch_object($result)) {
    $node_name = $row->{'system-name'};
    $board_family = $row->{'board-type'};
    $system_model = $row->{'system-model'};
    $node_list .= "$node_name/$board_family/$system_model ";

    if (strcasecmp($board_family, $vsim_family) === 0) {
        /* do not perform validation check for VSIM */
        continue;
    } elseif (strcasecmp($board_family, $vsa_family) === 0) {
        /* do not perform validation check for VSA */
        continue;
    } else {
        /* Extract the board family at the end of the 'board_type' string */
        if (($extracted_family = strrchr($board_family, ' ')) !== false) {
            $board_family = $extracted_family;
            $board_family = ltrim($board_family);
        }

        /* perform 'platform support validation' on actual hardware */
        if (array_key_exists($board_family, $supported_systems)) {
            /* supported board family */
            $tokenstr = strtok($supported_systems[$board_family], $token_seperator);

            /* check supported system models */
            while ($tokenstr !== false) {
                $tokenstr = trim($tokenstr);
                if (strcasecmp($system_model, $tokenstr) === 0) {
                    /* system model found in 'supported platform' list */
                    break;
                }
                $tokenstr = strtok($token_seperator);
            }

            if ($tokenstr === false) {
                /* did not found matched system model */
                if ($error_detected) {
                    $error_msg .= ", ";
                }
                $error_detected = true;
                $error_msg .= "$node_name ($system_model)";
            }
        } else {
            /* unsupported board family */
            if ($error_detected) {
                 $error_msg .= ", ";
            }
            $error_detected = true;
            $error_msg .= "$node_name ($system_model)";
        }
    }
}

if ($error_detected) {
    $error_msg .= ".";
    echo "<Alert>$error_msg</Alert>";
    echo "<Action>$action_msg</Action>";
    echo "<Advice>$advice_msg</Advice>";
    echo "<RC>error</RC>";
}
logDebugMessage("Nodes checked: $node_list", 'false');

exit;
?>
