<?php
//
//  Copyright (c) 2016 NetApp, Inc.
//  All rights reserved.
//

// VERSION="1.0"

//
// Each node with a mapped LUN must have a SAN LIF on the node and HA partner in the same Vserver
//

// burt1229806 revealed a performance issue with this script timing out with
// only 8k LUNs and 500 LIFs. As a fix, rather than using the CLI map table,
// we're going to manually join the vdiskMapTable with the vvolTable to get the
// list of nodes with maps.
//
// bcom_mgmt_object_resolver would be perfect for this, but dotsql doesn't
// support list fields, so we can't use it.


// Get partner node from storage_failover table
$result = dotsql_query("SELECT node-name,partner-name FROM storage_failover");
$HApartners = array();
while ($row = dotsql_fetch_object($result)) {
    $HApartners[$row->{'node-name'}] = $row->{'partner-name'};
}

if (empty($HApartners)) {
    // Short-circuit non-HA clusters. LIFs won't help when the only node with
    // access goes down. Do we let customers buy those?
    exit;
}

// We need the vsUuid to ID mapping since bcomd and vldb speak different
// vserver languages. We can also limit ourselves to only SAN vservers while
// we're at it.

// dotsql doesn't support "WHERE san-tgt-dev-id IS NOT NULL", so we do it
// manually.
$result = dotsql_query("SELECT vserver,id,uuid,san-tgt-dev-id FROM vserver");
$vservers = array();
while ($row = dotsql_fetch_object($result)) {
    if (property_exists($row, 'san-tgt-dev-id') && !empty($row->{'san-tgt-dev-id'})) {
        $vservers[$row->uuid] = [ 'name' => $row->vserver,
                                  'id' => $row->id,
                                  'lif_nodes' => array() ];
    }
}

// LIF tables should be much smaller than the map table, so we start by looking
// up the LIFs. If we happen to have a SAN LIF on every node in the cluster, we
// don't need to bother looking at maps.
$result = dotsql_query("SELECT server-uuid,home-node FROM iscsiLifTable");
while ($row = dotsql_fetch_object($result)) {
    // Create a hash of found nodes under each vserver so we can quickly
    // reference the LIF nodes later.
    $vsUuid = $row->{'server-uuid'};
    if (array_key_exists($vsUuid, $vservers)) {
        $vservers[$vsUuid]['lif_nodes'][$row->{'home-node'}] = 1;
    }
}

$result = dotsql_query("SELECT server-uuid,home-node FROM fcpLifTable");
while ($row = dotsql_fetch_object($result)) {
    $vsUuid = $row->{'server-uuid'};
    if (array_key_exists($vsUuid, $vservers)) {
        $vservers[$vsUuid]['lif_nodes'][$row->{'home-node'}] = 1;
    }
}

$printed = false;

foreach ($vservers as $vsUuid => $vserver) {
    // If we have LIFs on every node, we're good to go.
    $missing = false;
    foreach ($HApartners as $node => $partner) {
        if (!array_key_exists($node, $vserver['lif_nodes'])) {
            $missing = true;
            break;
        }
    }
    if (!$missing) {
        continue;
    }

    $lastMsid = 0;

    // Some node is missing LIFs. That's only a problem if the node has a
    // mapped LUN on it or its partner, so check which nodes have maps.
    $done = false;
    $volumes = array();
    while (!$done) {
        // We can potentially have 96k LUN maps in a single vserver. dotsql
        // loads the entire result into memory, so walk the list in smaller
        // chunks.
        $result = dotsql_query("SELECT vsUuid,volMsid FROM vdiskIgroupMap WHERE vsUuid=$vsUuid AND volMsid>$lastMsid LIMIT 1028");
        $done = dotsql_num_rows($result) < 1028;
        while ($row = dotsql_fetch_object($result)) {
            $lastMsid = $row->volMsid;
            $volumes[$lastMsid] = 1;
        }
    }

    // I'm guessing that the overhead of querying volumes one by one is higher
    // than just getting them all and processing it in memory here, so that's
    // what we're going with.
    $result = dotsql_query("SELECT volMsid,filerID FROM mgmtVolMsidTable WHERE vsID=" . $vserver['id']);

    // We're probably going to have many volumes on the same nodes, so we use
    // an associative array to remove duplicates as we go to avoid ballooning
    // up a huge list of the same nodes that we need to unique later.
    $missing_lif_nodes = array();
    while ($row = dotsql_fetch_object($result)) {
        if (array_key_exists($row->volMsid, $volumes) &&
            array_key_exists($row->filerID, $HApartners)) {
            // The volume contains a mapped LUN and is on an HA pair; make sure
            // both nodes have LIFs.
            if (!array_key_exists($row->filerID, $vserver['lif_nodes'])) {
                $missing_lif_nodes[$row->filerID] = 1;
            }

            $partner = $HApartners[$row->filerID];
            if (!array_key_exists($partner, $vserver['lif_nodes'])) {
                $missing_lif_nodes[$partner] = 1;
            }
        }
    }

    $vsName = $vserver['name'];
    foreach ($missing_lif_nodes as $node => $true) {
        if (!$printed) {
            $printed = true;
            echo "<Alert>Configure SAN LIFs on the following node-vserver pairs: $node: $vsName";
        } else {
            echo ", $node: $vsName";
        }
    }
}

if ($printed) {
    echo "</Alert>";
    echo "<Action>Run 'network interface create' with '-data-protocol' option equal to 'fcp' or 'iscsi' to create SAN LIF(s) on the node-vserver pairs shown</Action>";
    echo "<Advice>Ensure that any node with a mapped LUN and its HA partner have at least one SAN LIF each in the same vserver as the LUN</Advice>";
    echo "<RC>warning</RC>";
}

exit;
?>
