<?php
//
//  Copyright (c) 2014-2022 NetApp, Inc.
//  All rights reserved.
//

// VERSION="1.0"
// NDU Compatibility for snapmirror
//
// From 9.12.1 onwards DP relationships are not supported. This script will
// fail if DP relationship entries exist in the localDstSnapMirror table.
// If bootarg.sm_dp_enabled = true then it will not fail but throw a warning.
//
// For DP relationships, the script warns the user about the following scenarios:
// - Possible protection outage in case of intra-cluster relationships.
// - In case of inter-cluster relationships with remote source, the script warns
//   the user about protection outage if the remote cluster is not upgraded
//   first.
// - In case of inter-cluster relationships with both remote source and remote
//   destination, the script warns the user about possible cyclic dependencies
//   and protection outage that could occur because of it.
//
// For LS relationships, the script warns the user that some load sharing
// mirrors might not get updated while the cluster upgrade is in progress.

include 'mcc_util.php';

$intraClusterFound = false;
$remoteSrcFound = false;
$remoteDstNames;
$lsRelsFound = false;

$largeLunEnabled = false;
$largeLunBootargNode = "";
$warning_alert = "";
$warning_action = "";
$warning_advice = "";
$is_warning = 0;

$isSmDpEnabled = false;
$isDPDstRelFound = false;
$is_error = 0;
$error_alert = "";
$error_action = "";
$error_advice = "";


function logDebugMessage($message) {
    $dbgquery = "CALL add_scripting_log_message('validation_rules_snapmirror : $message','false')";
    $result = dotsql_query($dbgquery);
}

/**
 * Returns true if the bootarg is set to true, false otherwise.
 */
function isBootargTrue($val)
{
    return ((strcasecmp($val, "true") == 0) || ($val == "1"));
}

/**
 * Reads the kenv with the given name and returns its value.  If a kenv with
 * the given kenvNane does not exist, returns the empty string.
 */
function readKenv($kenvName)
{
    $value = "";
    global $largeLunBootargNode;

    $result = dotsql_query("SELECT node, value FROM debug_kenv WHERE variable = $kenvName");
    if ($row = dotsql_fetch_object($result)) {
        $value = $row->value;
	$largeLunBootargNode = $row->node;
    }

    return $value;
}

/*
 * look for DP SnapMirror relationships in the destination table
 */
function checkDpDstRelationships() {
    global $isDpDstRelFound;

    // check to see if there are any DP dst relationships at all
    $result = dotsql_query("SELECT * FROM localDstSnapMirror WHERE type = DP LIMIT 1");
    $result1 = dotsql_query("SELECT * FROM vserver_dr_conf WHERE snapmirror-type = DP LIMIT 1");
    if (dotsql_num_rows($result) || dotsql_num_rows($result1)) {
        logDebugMessage("DP SnapMirrors found");
        $isDpDstRelFound = true;
    } else {
        logDebugMessage("No DP SnapMirrors found");
    }
    return $isDpDstRelFound;
}

function checkDPRelations() {
    global $intraClusterFound;
    global $remoteSrcFound;
    global $remoteDstNames;

    // quick check to see if there are any DP relationships at all
    $result = dotsql_query("SELECT * FROM localSrcSnapMirror WHERE type = DP LIMIT 1");
    if (dotsql_num_rows($result) == 0) {
        logDebugMessage("DP SRC entries present");
	$result = dotsql_query("SELECT * FROM localDstSnapMirror WHERE type = DP LIMIT 1");
	$result1 = dotsql_query("SELECT * FROM vserver_dr_conf WHERE snapmirror-type = DP LIMIT 1");
	if (dotsql_num_rows($result) == 0 && dotsql_num_rows($result1) == 0) {
		logDebugMessage("No DP DST present");
	    // there are no SM relationships, so there is no more validation to
	    // be done; exit
	    return;
	}
	logDebugMessage("DP DST present");
    }

    // look for intra-cluster SM relationships. To do this find at least on
    // entry in localSrcSnapMirror with dstVsUuid of a local Vserver.

    $localVservers = dotsql_query("SELECT uuid FROM vserver WHERE type = data");
    while ($vserver = dotsql_fetch_object($localVservers)) {
	$result = dotsql_query(
		"SELECT *" .
		" FROM localSrcSnapMirror" .
		" WHERE dstVsUuid = " . $vserver->{'uuid'} . " AND type = DP" .
		" LIMIT 1");

	if (dotsql_num_rows($result) > 0) {
	    // we've found a intra-cluster DP relationship
	    logDebugMessage("Intra cluster DP found");
	    $intraClusterFound = true;
	    break;
	}
	logDebugMessage("No Intra cluster DP found");
    }
    unset($localVservers);

    // Now, we need to check if there is at least on inter-cluster SM relationship
    // with this cluster as the destination. For this, find the first relationship
    // in localDstSnapMirror with a remote vserver UUID.
    //
    // Also, we need to make a list of remote cluster that need to be upgraded first.
    // To do this, for each peered cluster, we need to check if there is at least
    // one entry in the localSrcSnapMirror for one of its constituent vservers.

    $peeredClusters = dotsql_query("SELECT cluster-uuid, cluster FROM clusterPeer_idTable");
    while ($cluster = dotsql_fetch_object($peeredClusters)) {
	$peeredVservers = dotsql_query(
	    "SELECT peer-vserver-uuid".
	    " FROM vserverPeer_rdb_tbl".
	    " WHERE peer-clusterID = " . $cluster->{'cluster-uuid'} .
	    " AND applications = snapmirror");

	// a remote vserver can be peered to more than one local server, so it can
	// appear several times in the query above. $processedVservers will be used
	// to avoid processing a remote vserver more than once.
	$processedVservers;

	$remoteDstFound = false;
	// for each remote vserver, find at least one relationship with remote
	// source and one relationship with remote destination.
	while (($remoteDstFound == false || $remoteSrcFound == false) &&
		$vserver = dotsql_fetch_object($peeredVservers)) {

	    if (array_key_exists($vserver->{'peer-vserver-uuid'},
				 $processedVservers)) {
		// we have checked for this vserver already, so move on to the next
		// vserver
		continue;
	    } else {
		$processedVservers[$vserver->{'peer-vserver-uuid'}] = 1;
	    }

	    if ($remoteDstFound == false) {
		// check for inter-cluster relationships with remote destination
		$result = dotsql_query(
		    "SELECT *" .
		    " FROM localSrcSnapMirror" .
		    " WHERE dstVsUuid = " . $vserver->{'peer-vserver-uuid'} .
			" AND type = DP" .
		    " LIMIT 1");

		if (dotsql_num_rows($result) > 0) {
		    // add the cluster to the list of clusters that need to be
		    // upgraded first.
		    if (!empty($remoteDstNames)) {
			$remoteDstNames = $remoteDstNames . ", ";
		    }
		    $remoteDstNames = $remoteDstNames . $cluster->{cluster};
		    $remoteDstFound = true;
		}
	    }

	    if ($remoteSrcFound == false) {
		// check for inter-cluster relationships with remote source
		$result = dotsql_query(
		    "SELECT *" .
		    " FROM localDstMirrorBySrcVserver" .
		    " WHERE srcVsUuid = " . $vserver->{'peer-vserver-uuid'} .
			" AND type = DP" .
		    " LIMIT 1");

		if (dotsql_num_rows($result) > 0) {
		    // we have found a record!
		    $remoteSrcFound = true;
		}
	    }
	}
    }
}

function checkLSRelations() {
    global $lsRelsFound;

    $result = dotsql_query("SELECT * FROM snapmirrorTable WHERE type = LS LIMIT 1");
    $lsRelsFound = false;
    if (dotsql_num_rows($result) > 0) {
	// we've found a LS relationship
	$lsRelsFound = true;
        logDebugMessage("LS SnapMirrors found");
    } else {
        logDebugMessage("No LS SnapMirrors found");
    }
}

/**
 * Returns true if Large LUN bootarg is set, false otherwise.
 */
function isLargeLunEnabled()
{
    global $largeLunEnabled;
    global $largeLunBootargNode;
    $bootargName = "bootarg.init.large_lun_enabled";

    $result = dotsql_query("SELECT node, value FROM debug_kenv WHERE variable = $bootargName");
    while ($row = dotsql_fetch_object($result)) {
        $value = $row->value;
	if (!empty($largeLunBootargNode)) {
	     $largeLunBootargNode = $largeLunBootargNode .",".$row->node;
	} else {
	     $largeLunBootargNode = $row->node;
	}
	if ($largeLunEnabled == false) {
	     $largeLunEnabled = isBootargTrue($row->value);
	}
    }
    return $largeLunEnabled;
}

/**
 * Returns true if SnapMirror DP bootarg is set to true, false otherwise.
 */
function isSnapMirrorDpEnabled()
{
    global $isSmDpEnabled;
    $bootargName = "bootarg.sm_dp_enabled";

    $result = dotsql_query(
        "SELECT value FROM debug_kenv WHERE variable = $bootargName AND value = true LIMIT 1");
    if (dotsql_num_rows($result)) {
        logDebugMessage("bootarg.sm_dp_enabled true");
	$isSmDpEnabled = true;
    } else {
        logDebugMessage("bootarg.sm_dp_enabled NOT true");
    }
    return $isSmDpEnabled;
}

/**
 * Print the warning text.
 */
function printWarning()
{
    global $warning_alert;
    global $warning_action;
    global $warning_advice;

    echo "<Alert>$warning_alert</Alert>";
    echo "<Action>$warning_action</Action>";
    echo "<Advice>$warning_advice</Advice>";
    echo "<RC>warning</RC>";
}


/**
 * Print the error text.
 */
function printError()
{
    global $error_alert;
    global $error_action;
    global $error_advice;

    echo "<Alert>$error_alert</Alert>";
    echo "<Action>$error_action</Action>";
    echo "<Advice>$error_advice</Advice>";
    echo "<RC>error</RC>";
}


/***************************
 * BEGIN
 ***************************/

if (isLargeLunEnabled()) {
    $warning_alert = "Bootarg to enable support for Large LUNs is set on the node(s) ".$largeLunBootargNode."." ;
    $warning_action = "If the bootarg was set as a workaround, please unset the Large LUN bootarg before updating any nodes. To unconfigure, run the (privilege: advanced) 'run -node ".$largeLunBootargNode." \"bootargs unset bootarg.init.large_lun_enabled\"' command.";
    $warning_advice = "Please unset the Large LUN bootarg before updating any nodes.";
    $is_warning++;
}

// All the validations only apply for major upgrades
$options = getoptreq('', array("major:", "downgrade::"));
$majorupgrade = $options["major"];
if($majorupgrade == "false") {
    if($is_warning > 0){
        printWarning();
    }
    // exit here because this is a minor upgrade
    exit;
}

# If DP DST relationships exist on cluster,
# if bootarg is NOT set
#   * throw ERROR
# else
#  * throw WARNING
# Do other DP relationships checks if bootarg is set
$alert = "";
$action = "";
$advice = "";
if (checkDpDstRelationships()) {
    $alert = "DP SnapMirror relationships exist on this cluster.";
    $action = "Identify the DP relationships using the command ".
              "\"snapmirror show -type DP\". ".
              "Refer to https://docs.netapp.com/us-en/ontap/data-protection/convert-snapmirror-version-flexible-task.html ".
              "to convert DP to XDP relationship or delete and release ".
              "the relationships using the ".
              "\"snapmirror delete\" and \"snapmirror release\" commands.";
    $advice = "Convert DP to XDP or Delete and Release all DP SnapMirror ".
              "relationships before the upgrade.";
    if (isSnapMirrorDpEnabled()) {
        logDebugMessage("DP bootarg enabled");
	if($is_warning > 0)
	{
            $warning_alert = $warning_alert." ".$alert;
            $warning_action = $warning_action." ".$action;
            $warning_advice = $warning_advice." ".$advice;
	} else {
            $warning_alert = $alert;
            $warning_action = $action;
            $warning_advice = $advice;
	}
        $is_warning++;

        # do more checks for DP relationships because user can continue to have DP
        checkDPRelations();
    } else {
        logDebugMessage("DP bootarg NOT enabled");
        // prevent the upgrade and throw ERROR
	if($is_error > 0)
	{
            $error_alert = $error_alert." ".$alert;
            $error_action = $error_action." ".$action;
            $error_advice = $error_advice." ".$advice;
	} else {
            $error_alert = $alert;
            $error_action = $action;
            $error_advice = $advice;
	}
        $is_error++;
    }
} else {
    # check for DP relationships
    checkDPRelations();
}

# check for LS relationships
checkLSRelations();

if ($intraClusterFound == false &&
    empty($remoteDstNames) &&
    $lsRelsFound == false &&
    $isDpDstRelFound == false &&
    $largeLunEnabled == false)
{
    // no warnings; it's all good
    exit;
}

// frame the action/advice message:
$alert = "";
$advice = "";
$action = "";
if ($intraClusterFound == true) {
    // warn the user about protection outage for intra-cluster relationships
    $advice = "For intra-cluster DP SnapMirror relationships, you may lose" .
	      " protection for the duration of the upgrade.";

    $alert = "DP";
}

if (!empty($remoteDstNames)) {
    // advice the user to upgrade the destination clusters first.
    if (!empty($advice)) {
	$advice = $advice . " ";
    }
    $advice = $advice . "For inter-cluster DP SnapMirror relationships, upgrade" .
	      " the following destination clusters first before upgrading the" .
	      " current cluster: " .  $remoteDstNames . "." .
	      " If the current cluster is upgraded first, you will lose" .
	      " protection until the destination clusters are upgraded, too.";

    $alert = "DP";
}

if (!empty($remoteDstNames) && $remoteSrcFound == true) {
    // warn the user about possible cyclic relationships
    if (!empty($advice)) {
	$advice = $advice . " ";
    }
    $advice = $advice . "If the cluster hosts both source and destination" .
		" volumes for different sets of DP SnapMirror relationships," .
		" you will lose data protection for all the relationships for" .
		" which the current cluster is the source until the destination" .
		" cluster is upgraded, too.";

    $alert = "DP";
}

if (true == $lsRelsFound) {
    if (!empty($advice)) {
	$advice = $advice . " ";
    }
    $advice = $advice . "For LS SnapMirror relationships, mirror copies might" .
		" not get updated while the cluster is being upgraded." .
		" Mirroring will resume once all the nodes in the cluster have" .
		" been upgraded. It is recommended that all load-sharing mirror" .
		" copies be updated manually before and after the upgrade" .
		" process using the \"snapmirror update-ls-set\" command.";

    if (!empty($alert)) {
        $alert = $alert . " and LS";
    } else {
        $alert = "LS";
    }
}

$alert = $alert . " SnapMirror relationships detected.";
if($is_warning > 0)
{
	$warning_alert = $warning_alert." ".$alert;
	$warning_action = $warning_action." ".$action;
	$warning_advice = $warning_advice." ".$advice;
} else {
	$warning_alert = $alert;
	$warning_action = $action;
	$warning_advice = $advice;
}

// dump out the messages
if($is_error > 0){
	printError();
}
if($is_warning > 0){
	printWarning();
}
exit;
?>
